@@ -4,7 +4,7 @@ static GtkWidget *statusbar;
 static GtkWidget *entry;
 static GtkActionGroup *action_group;
 
-static GtkIEEmbed *create_new_embed (GtkNotebook *notebook, const gchar *url);
+static GtkIEEmbed *create_new_embed (GtkNotebook *notebook);
 
 static GtkIEEmbed *
 get_current_embed (GtkNotebook *notebook)
@@ -16,7 +16,7 @@ get_current_embed (GtkNotebook *notebook)
 static void
 open_new_tab_action (GtkAction *action, GtkNotebook *notebook)
 {
-    create_new_embed (GTK_NOTEBOOK (notebook), NULL);
+    create_new_embed (GTK_NOTEBOOK (notebook));
 }
 
 static void
@@ -385,7 +385,7 @@ cb_net_stop (GtkIEEmbed *ie, gpointer data)
 static gboolean
 cb_new_window (GtkIEEmbed *ie, GtkIEEmbed **newie, GtkWidget *notebook)
 {
-    *newie = create_new_embed (GTK_NOTEBOOK (notebook), NULL);
+    *newie = create_new_embed (GTK_NOTEBOOK (notebook));
 
     return FALSE;
 }
@@ -454,7 +454,7 @@ cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 }
 
 static GtkIEEmbed *
-create_new_embed (GtkNotebook *notebook, const gchar *url)
+create_new_embed (GtkNotebook *notebook)
 {
     GtkWidget *embed, *label;
 
@@ -469,21 +469,39 @@ create_new_embed (GtkNotebook *notebook, const gchar *url)
     g_signal_connect (embed, "new-window", G_CALLBACK (cb_new_window), notebook);
     g_signal_connect (embed, "close-window", G_CALLBACK (cb_close_window), notebook);
 
-    gtk_ie_embed_load_url (GTK_IE_EMBED (embed), url);
-
     return GTK_IE_EMBED (embed);
 }
 
-int
-main (int argc, char *argv[])
+static void
+append_new_embed_with_url (GtkNotebook *notebook, const gchar *url)
+{
+    GtkIEEmbed *embed;
+
+    embed = create_new_embed (GTK_NOTEBOOK (notebook));
+    gtk_ie_embed_load_url (embed, url);
+}
+
+static void
+append_new_welcome_embed (GtkNotebook *notebook)
+{
+    GtkIEEmbed *embed;
+    gchar contents[] = "<html><title>Welcome</title>"
+                       "<body>This page is rendered by GtkIEEmbed.</body>"
+                       "</html>";
+
+    embed = create_new_embed (GTK_NOTEBOOK (notebook));
+
+    gtk_ie_embed_load_html_from_string (embed, contents);
+}
+
+static void
+create_main_window (void)
 {
     GtkWidget *window;
-    GtkWidget *vbox, *hbox;
     GtkWidget *notebook;
+    GtkWidget *vbox, *hbox;
     GtkUIManager *merge;
 
-    gtk_init (&argc, &argv);
-
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_default_size (GTK_WINDOW (window), 600, 400);
     g_signal_connect (window, "delete-event",
@@ -540,8 +558,17 @@ main (int argc, char *argv[])
 
     gtk_widget_show (window);
 
-    create_new_embed (GTK_NOTEBOOK (notebook), "http://www.gnome.org/");
-    create_new_embed (GTK_NOTEBOOK (notebook), "http://www.google.com/");
+    append_new_welcome_embed (GTK_NOTEBOOK (notebook));
+    append_new_embed_with_url (GTK_NOTEBOOK (notebook),
+                               "http://www.gnome.org/");
+}
+
+int
+main (int argc, char *argv[])
+{
+    gtk_init (&argc, &argv);
+
+    create_main_window ();
 
     gtk_main ();
 
@@ -550,6 +577,30 @@ main (int argc, char *argv[])
     return 0;
 }
 
+#ifdef _WINDOWS
+#include <windows.h>
+int WINAPI
+WinMain (HINSTANCE hInstance,
+         HINSTANCE hPrevInstance,
+         LPSTR     lpszCmdLine,
+         int       nCmdShow)
+{
+    char **argv;
+    int i, argc, ret = -1;
+
+    argc = __argc;
+    argv = g_malloc ((argc + 1) * sizeof (char *));
+    for (i = 0; i < argc; i++)
+        argv[i] = g_locale_to_utf8 (__argv[i], -1, NULL, NULL, NULL);
+    argv[i] = NULL;
+    ret = main (argc, argv);
+    for (i = 0; i < argc; i++)
+        g_free (argv[i]);
+    g_free (argv);
+    return ret;
+}
+#endif /* _WINDOWS */
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
