@@ -2,20 +2,18 @@
 /*
  *  Copyright (C) 2007 Hiroyuki Ikezoe  <poincare@ikezoe.net>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
+ *  This library is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this program; if not, write to the
- *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- *  Boston, MA  02111-1307  USA
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -44,6 +42,7 @@ enum {
     DOM_MOUSE_UP_SIGNAL,
     DOM_MOUSE_CLICK_SIGNAL,
     SELECTION_CHANGED_SIGNAL,
+    FAVICON_SIGNAL,
     LAST_SIGNAL
 };
 
@@ -234,6 +233,16 @@ gtk_ie_embed_class_init (GtkIEEmbedClass *klass)
                 g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);
 
+    gtk_ie_embed_signals[FAVICON_SIGNAL]
+        = g_signal_new ("favicon",
+                GTK_TYPE_IE_EMBED,
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET(GtkIEEmbedClass, favicon),
+                NULL, NULL,
+                g_cclosure_marshal_VOID__OBJECT,
+                G_TYPE_NONE, 1,
+                GDK_TYPE_PIXBUF);
+
     g_type_class_add_private (gobject_class, sizeof (GtkIEEmbedPriv));
 }
 
@@ -577,14 +586,14 @@ gtk_ie_embed_set_font_size (GtkIEEmbed *ie, GtkIEEmbedFontSize size)
     if (!priv->bridge)
         return;
 
-    _ie_bridge_set_font_size (priv->bridge, (guint) size);
+    _ie_bridge_set_font_size (priv->bridge, size);
 }
 
 GtkIEEmbedFontSize
 gtk_ie_embed_get_font_size (GtkIEEmbed *ie)
 {
     GtkIEEmbedFontSize size;
-    guint usize;
+    gint usize;
     GtkIEEmbedPriv *priv = GTK_IE_EMBED_GET_PRIVATE (ie);
 
     if (!priv->bridge)
@@ -790,11 +799,11 @@ gtk_ie_embed_dom_event_target_free (GtkIEEmbedDOMEventTarget *target)
             g_free (attr->name);
         if (attr->value)
             g_free (attr->value);
-        g_free (attr);
+        g_slice_free (GtkIEEmbedDOMEventTargetAttribute, attr);
     }
     g_list_free (list);
 
-    g_free (target);
+    g_slice_free (GtkIEEmbedDOMEventTarget, target);
 }
 
 gboolean
@@ -803,8 +812,7 @@ _gtk_ie_embed_new_window (GtkIEEmbed *ie, IEBridge **newbridge)
     gboolean cancel = FALSE;
     GtkIEEmbed *newie = NULL;
 
-    g_signal_emit_by_name (G_OBJECT (ie), "new-window",
-                           &newie, &cancel);
+    g_signal_emit_by_name (ie, "new-window", &newie, &cancel);
 
     if (newie) {
         GtkIEEmbedPriv *priv = GTK_IE_EMBED_GET_PRIVATE (newie);
