@@ -2,20 +2,18 @@
 /*
  *  Copyright (C) 2007 Hiroyuki Ikezoe  <poincare@ikezoe.net>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
+ *  This library is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this program; if not, write to the
- *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- *  Boston, MA  02111-1307  USA
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,6 +29,7 @@
 #include "gtk-ie-embed-private.h"
 #include "ie-browser-event-dispatcher.h"
 #include "ie-document-event-dispatcher.h"
+#include "ie-utils.h"
 
 enum {
     PROP_0,
@@ -56,7 +55,7 @@ struct _IEBridgePriv
 
 G_DEFINE_TYPE (IEBridge, _ie_bridge, G_TYPE_OBJECT)
 
-typedef HRESULT (*ATLAXGETCONTROL) (HWND, IUnknown **);
+typedef HRESULT (__stdcall *ATLAXGETCONTROL) (HWND, IUnknown **);
 static HMODULE hModule = NULL;
 static ATLAXGETCONTROL AtlAxGetControl = NULL;
 
@@ -97,7 +96,7 @@ _ie_bridge_class_init (IEBridgeClass *klass)
     g_type_class_add_private (gobject_class, sizeof (IEBridgePriv));
 }
 
-typedef BOOL (*ATLAXWININIT) (void);
+typedef BOOL (__stdcall *ATLAXWININIT) (void);
 
 static gboolean
 atl_initialize (void)
@@ -106,7 +105,7 @@ atl_initialize (void)
 
     if (!atl_initialized) {
         ATLAXWININIT AtlAxWinInit = NULL;
-        hModule = LoadLibrary ("atl.dll");
+        hModule = LoadLibraryA ("atl.dll");
         if (!hModule)
             return FALSE;
         CoInitialize (NULL);
@@ -167,10 +166,10 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
     priv = IE_BRIDGE_GET_PRIVATE (object);
     hInstance = GetModuleHandle (NULL);
     hparent = (HWND) GDK_WINDOW_HWND (gtk_widget_get_parent_window (priv->widget));
-    hwnd = CreateWindow ("AtlAxWin", "Shell.Explorer.2", WS_CHILD | WS_VISIBLE,
-                         priv->widget->allocation.x, priv->widget->allocation.y,
-                         priv->widget->allocation.width, priv->widget->allocation.height,
-                         hparent, (HMENU)0, hInstance, NULL);
+    hwnd = CreateWindowA ("AtlAxWin", "Shell.Explorer.2", WS_CHILD | WS_VISIBLE,
+                          priv->widget->allocation.x, priv->widget->allocation.y,
+                          priv->widget->allocation.width, priv->widget->allocation.height,
+                          hparent, (HMENU)0, hInstance, NULL);
     if (hwnd) {
         IUnknown *iUnknown;
 
@@ -305,19 +304,18 @@ void
 _ie_bridge_load_url (IEBridge *ie, const gchar *url)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
-    gunichar2 *utf16_url;
+    BSTR bstrUrl;
 
     if (!priv->web_browser)
         return;
 
-    utf16_url = g_utf8_to_utf16 (url, -1, NULL, NULL, NULL);
-    if (utf16_url) {
-        priv->web_browser->Navigate ((OLECHAR *) utf16_url, NULL, NULL, NULL, NULL);
-        g_free (utf16_url);
+    bstrUrl = _ie_utils_utf8_to_BSTR (url);
+    if (bstrUrl) {
+        priv->web_browser->Navigate (bstrUrl, NULL, NULL, NULL, NULL);
+        SysFreeString (bstrUrl);
     }
 }
 
-
 void
 _ie_bridge_reload (IEBridge *ie, RefreshConstants flag)
 {
@@ -385,7 +383,7 @@ _ie_bridge_get_location (IEBridge *ie)
         BSTR URL;
         priv->web_browser->get_LocationURL (&URL);
         if (URL) {
-            location = g_utf16_to_utf8 ((gunichar2 *) URL, -1, NULL, NULL, NULL);
+            location = _ie_utils_BSTR_to_utf8 (URL);
             SysFreeString (URL);
         }
     }
@@ -403,7 +401,7 @@ _ie_bridge_get_title (IEBridge *ie)
         BSTR Name;
         priv->web_browser->get_LocationName (&Name);
         if (Name) {
-            title = g_utf16_to_utf8 ((gunichar2 *) Name, -1, NULL, NULL, NULL);
+            title = _ie_utils_BSTR_to_utf8 (Name);
             SysFreeString (Name);
         }
     }
@@ -421,7 +419,7 @@ _ie_bridge_get_charset (IEBridge *ie)
         BSTR Charset = NULL;
         priv->current_document->get_charset (&Charset);
         if (Charset) {
-            charset = g_utf16_to_utf8 ((gunichar2 *) Charset, -1, NULL, NULL, NULL);
+            charset = _ie_utils_BSTR_to_utf8 (Charset);
             SysFreeString (Charset);
         }
     }
@@ -435,7 +433,7 @@ _ie_bridge_set_charset (IEBridge *ie, const gchar *charset)
 
     if (priv->web_browser) {
         BSTR Charset;
-        Charset = (BSTR) g_utf8_to_utf16 (charset, -1, NULL, NULL, NULL);
+        Charset = _ie_utils_utf8_to_BSTR (charset);
         if (Charset) {
             priv->current_document->put_charset (Charset);
             SysFreeString (Charset);
@@ -522,7 +520,7 @@ _ie_bridge_can_paste_clipboard (IEBridge *ie)
 }
 
 void
-_ie_bridge_set_font_size (IEBridge *ie, guint size)
+_ie_bridge_set_font_size (IEBridge *ie, gint size)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
@@ -536,7 +534,7 @@ _ie_bridge_set_font_size (IEBridge *ie, guint size)
     }
 }
 
-guint
+gint
 _ie_bridge_get_font_size (IEBridge *ie)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
@@ -549,6 +547,7 @@ _ie_bridge_get_font_size (IEBridge *ie)
                                    NULL, &zoom);
         return zoom.lVal;
     }
+    return -1;
 }
 
 void
@@ -608,7 +607,7 @@ _ie_bridge_get_last_modified (IEBridge *ie)
         return NULL;
     doc->get_lastModified (&mTime);
     if (mTime) {
-        last_modified = g_utf16_to_utf8 ((gunichar2 *) mTime, -1, NULL, NULL, NULL);
+        last_modified = _ie_utils_BSTR_to_utf8 (mTime);
         SysFreeString (mTime);
     }
 
@@ -645,9 +644,7 @@ gchar *
 _ie_bridge_get_selected_text (IEBridge *ie)
 {
     gchar *text = NULL;
-    IHTMLSelectionObject *selection;
     IHTMLTxtRange *text_range = NULL;
-    gboolean ret;
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
     if (!priv->web_browser)
@@ -656,11 +653,11 @@ _ie_bridge_get_selected_text (IEBridge *ie)
         return NULL;
     if (get_selected_text_range (priv->current_document, &text_range) &&
         text_range) {
-        BSTR selected_text;
-        text_range->get_text (&selected_text);
-        if (selected_text) {
-            text = g_utf16_to_utf8 ((gunichar2 *) selected_text, -1, NULL, NULL, NULL);
-            SysFreeString (selected_text);
+        BSTR selectedText = NULL;
+        text_range->get_text (&selectedText);
+        if (selectedText) {
+            text = _ie_utils_BSTR_to_utf8 (selectedText);
+            SysFreeString (selectedText);
         }
         text_range->Release();
     }
@@ -701,41 +698,42 @@ _ie_bridge_find_string (IEBridge *ie,
     if (get_selected_text_range (doc, &selected_range) && selected_range) {
         long ActualCount;
         gunichar2 *unit, *how;
-        unit = g_utf8_to_utf16 ("character", -1, NULL, NULL, NULL);
+        BSTR Unit = NULL, How = NULL;
+        Unit = _ie_utils_utf8_to_BSTR ("character");
         if (forward) {
-            how = g_utf8_to_utf16 ("EndToEnd", -1, NULL, NULL, NULL);
-            selected_range->moveStart ((OLECHAR *) unit, 1, &ActualCount);
+            How = _ie_utils_utf8_to_BSTR ("EndToEnd");
+            selected_range->moveStart (Unit, 1, &ActualCount);
         } else {
-            how = g_utf8_to_utf16 ("StartToStart", -1, NULL, NULL, NULL);
-            selected_range->moveEnd ((OLECHAR *) unit, -1, &ActualCount);
+            How = _ie_utils_utf8_to_BSTR ("StartToStart");
+            selected_range->moveEnd (Unit, -1, &ActualCount);
         }
-        selected_range->setEndPoint ((OLECHAR *) how, text_range);
+        selected_range->setEndPoint (How, text_range);
         text_range->Release ();
         text_range = selected_range;
-        g_free (unit);
-        g_free (how);
+        SysFreeString (Unit);
+        SysFreeString (How);
     }
 
     if (text_range) {
-        gunichar2 *utf16_string;
-        utf16_string = g_utf8_to_utf16 (string, -1, NULL, NULL, NULL);
-        if (utf16_string) {
+        BSTR bstr = NULL;
+        bstr = _ie_utils_utf8_to_BSTR (string);
+        if (bstr) {
             VARIANT_BOOL vBool;
             gint direction = forward ? 1 : -1;
-            text_range->findText ((OLECHAR *) utf16_string, direction, 0, &vBool);
+            text_range->findText (bstr, direction, 0, &vBool);
             if (vBool == VARIANT_TRUE) {
                 text_range->select ();
                 ret = TRUE;
             } else if (selected_range && wrap) {
                 text_range->Release ();
                 body->createTextRange (&text_range);    
-                text_range->findText ((OLECHAR *) utf16_string, direction, 0, &vBool);
+                text_range->findText (bstr, direction, 0, &vBool);
                 if (vBool == VARIANT_TRUE) {
                     text_range->select ();
                     ret = TRUE;
                 }
             }
-            g_free (utf16_string);
+            SysFreeString (bstr);
         }
         text_range->Release ();
     }
@@ -750,64 +748,75 @@ _ie_bridge_find_string (IEBridge *ie,
 void
 _ie_bridge_load_html_from_string (IEBridge *ie, const gchar *string)
 {
-    IStream *stream;
-    IPersistStreamInit *persist_stream = NULL;
     IHTMLDocument2 *doc = NULL;
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
+    BSTR bstr = NULL;
+    VARIANT *param;
+    SAFEARRAY *sfArray;
 
     if (!priv->web_browser)
         return;
 
+    _ie_bridge_load_url (ie, "about:blank");
     priv->web_browser->get_Document ((IDispatch **) &doc);
     if (!doc)
         return;
 
-    CreateStreamOnHGlobal ((HGLOBAL) string, TRUE, &stream);
-    if (stream) {
-        doc->QueryInterface (IID_IPersistStreamInit, (void **) persist_stream);
-        if (persist_stream) {
-            persist_stream->InitNew ();
-            persist_stream->Load (stream);
-            persist_stream->Release ();
-        }
-        stream->Release ();
+    bstr = _ie_utils_utf8_to_BSTR (string);
+    if (!bstr)
+        return;
+
+    sfArray = SafeArrayCreateVector (VT_VARIANT, 0, 1);
+    if (!sfArray) {
+        SysFreeString (bstr);
+        return;
     }
 
-    doc->Release ();
+    SafeArrayAccessData (sfArray, (LPVOID*) & param);
+    param->vt = VT_BSTR;
+    param->bstrVal = bstr;
+    SafeArrayUnaccessData (sfArray);
+    doc->write (sfArray);
+
+    SysFreeString (bstr);
+    SafeArrayDestroy (sfArray);
 }
 
 void
-_ie_bridge_title_changed (IEBridge *ie, gunichar2 *title)
+_ie_bridge_title_changed (IEBridge *ie, BSTR title)
 {
     gchar *utf8_title;
-    IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
-    utf8_title = g_utf16_to_utf8 ((gunichar2 *) title, -1, NULL, NULL, NULL);
+
+    utf8_title = _ie_utils_BSTR_to_utf8 (title);
     if (utf8_title) {
-        g_signal_emit_by_name (G_OBJECT (priv->widget), "title", utf8_title, 0);
+        IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
+        g_signal_emit_by_name (priv->widget, "title", utf8_title);
         g_free (utf8_title);
     }
 }
 
 void
-_ie_bridge_location_changed (IEBridge *ie, gunichar2 *location)
+_ie_bridge_location_changed (IEBridge *ie, BSTR location)
 {
     gchar *utf8_location;
-    IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
-    utf8_location = g_utf16_to_utf8 ((gunichar2 *) location, -1, NULL, NULL, NULL);
+
+    utf8_location = _ie_utils_BSTR_to_utf8 (location);
     if (utf8_location) {
-        g_signal_emit_by_name (G_OBJECT (priv->widget), "location", utf8_location, 0);
+        IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
+        g_signal_emit_by_name (priv->widget, "location", utf8_location);
         g_free (utf8_location);
     }
 }
 
 void
-_ie_bridge_status_text_change (IEBridge *ie, gunichar2 *text)
+_ie_bridge_status_text_change (IEBridge *ie, BSTR text)
 {
     gchar *utf8_text;
-    IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
-    utf8_text = g_utf16_to_utf8 ((gunichar2 *) text, -1, NULL, NULL, NULL);
+
+    utf8_text = _ie_utils_BSTR_to_utf8 (text);
     if (utf8_text) {
-        g_signal_emit_by_name (G_OBJECT (priv->widget), "text", utf8_text, 0);
+        IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
+        g_signal_emit_by_name (priv->widget, "status-text", utf8_text);
         g_free (utf8_text);
     }
 }
@@ -886,7 +895,7 @@ _ie_bridge_close_window (IEBridge *ie, gboolean is_child)
     gboolean cancel = FALSE;
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
-    g_signal_emit_by_name (G_OBJECT (priv->widget), "close-window",
+    g_signal_emit_by_name (priv->widget, "close-window",
                            &cancel);
     return cancel;
 }
@@ -896,7 +905,7 @@ _ie_bridge_net_start (IEBridge *ie)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
-    g_signal_emit_by_name (G_OBJECT (priv->widget), "net-start", 0);
+    g_signal_emit_by_name (priv->widget, "net-start");
 }
 
 void
@@ -904,7 +913,7 @@ _ie_bridge_net_stop (IEBridge *ie)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
-    g_signal_emit_by_name (G_OBJECT (priv->widget), "net-stop", 0);
+    g_signal_emit_by_name (priv->widget, "net-stop");
 }
 
 void
@@ -912,7 +921,7 @@ _ie_bridge_progress_change (IEBridge *ie, glong current_progress, glong max_prog
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
-    g_signal_emit_by_name (G_OBJECT (priv->widget), "progress",
+    g_signal_emit_by_name (priv->widget, "progress",
                            current_progress, max_progress);
 
 }
@@ -940,7 +949,7 @@ _set_dom_event_target_attributes (GtkIEEmbedDOMEventTarget *target, IHTMLElement
         for (i = 0; i < length; i++) {
             IDispatch *disp = NULL;
             IHTMLDOMAttribute *dom_attr = NULL;
-            BSTR nodeName
+            BSTR nodeName;
             VARIANT nodeValue;
             gchar *utf8_name, *utf8_value;
             VARIANT v;
@@ -950,35 +959,41 @@ _set_dom_event_target_attributes (GtkIEEmbedDOMEventTarget *target, IHTMLElement
             disp->QueryInterface (IID_IHTMLDOMAttribute, (IHTMLDOMAttribute **) &dom_attr);
             dom_attr->get_nodeName (&nodeName);
             dom_attr->get_nodeValue (&nodeValue);
-            utf8_name = g_utf16_to_utf8 ((gunichar2 *) nodeName, -1, NULL, NULL, NULL);
-            if (nodeValue.vt == VT_BSTR && nodeValue.bstrVal) {
-                utf8_value = g_utf16_to_utf8 ((gunichar2 *) nodeValue.bstrVal, -1, NULL, NULL, NULL);
-            }
-            attr = g_new0 (GtkIEEmbedDOMEventTargetAttribute, 1);
+            utf8_name = _ie_utils_BSTR_to_utf8 (nodeName);
+            if (nodeValue.vt == VT_BSTR && nodeValue.bstrVal)
+                utf8_value = _ie_utils_BSTR_to_utf8 (nodeValue.bstrVal);
+
+            attr = g_slice_new0 (GtkIEEmbedDOMEventTargetAttribute);
             attr->name = utf8_name;
             attr->value = utf8_value;
             target_attribute_list = g_list_prepend (target_attribute_list, attr);
+            dom_attr->Release ();
             disp->Release ();
             SysFreeString (nodeName);
         }
+        collection->Release ();
     }
+    node->Release ();
 #else
     /* workaround */
-    VARIANT attr_value;
-    gunichar2 *attr_name;
+    VARIANT attrValue;
+    BSTR attrName = NULL;
 
-    attr_name = g_utf8_to_utf16 ("href", -1, NULL, NULL, NULL);
-    target_element->getAttribute ((OLECHAR *) attr_name, 0, &attr_value);
-    if (attr_value.vt == VT_BSTR && attr_value.bstrVal) {
+    attrName = _ie_utils_utf8_to_BSTR ("href");
+    if (!attrName)
+        return;
+
+    target_element->getAttribute (attrName, 0, &attrValue);
+    if (attrValue.vt == VT_BSTR && attrValue.bstrVal) {
         gchar *value = NULL;
-        attr = g_new0 (GtkIEEmbedDOMEventTargetAttribute, 1);
+        attr = g_slice_new0 (GtkIEEmbedDOMEventTargetAttribute);
         attr->name = g_strdup ("href");
-        value = g_utf16_to_utf8 ((gunichar2 *) attr_value.bstrVal, -1, NULL, NULL, NULL);
+        value = _ie_utils_BSTR_to_utf8 (attrValue.bstrVal);
         if (value)
             attr->value = value;
         target->attribute_list = g_list_prepend (target->attribute_list, attr);
     }
-    g_free (attr_name);
+    SysFreeString (attrName);
 #endif /* HAVE_IHTMLNODE */
 }
 
@@ -986,16 +1001,16 @@ static GtkIEEmbedDOMEventTarget *
 _ie_bridge_create_dom_event_target (IHTMLElement *target_element)
 {
     GtkIEEmbedDOMEventTarget *target;
-    BSTR tag_name = NULL;
+    BSTR tagName = NULL;
 
-    target = g_new0 (GtkIEEmbedDOMEventTarget, 1);
+    target = g_slice_new0 (GtkIEEmbedDOMEventTarget);
 
-    target_element->get_tagName (&tag_name);
-    if (tag_name) {
+    target_element->get_tagName (&tagName);
+    if (tagName) {
         gchar *utf8_tag_name;
-        utf8_tag_name = g_utf16_to_utf8 ((gunichar2 *) tag_name, -1, NULL, NULL, NULL);
+        utf8_tag_name = _ie_utils_BSTR_to_utf8 (tagName);
         target->name = utf8_tag_name;
-        SysFreeString (tag_name);
+        SysFreeString (tagName);
     }
 
     _set_dom_event_target_attributes (target, target_element);
@@ -1013,7 +1028,7 @@ _ie_bridge_create_dom_mouse_event (IEBridge *ie)
 
     gdk_window_get_pointer (priv->widget->window, &x, &y, &mask);
 
-    event = g_new0 (GtkIEEmbedDOMMouseEvent, 1);
+    event = g_slice_new0 (GtkIEEmbedDOMMouseEvent);
 
     event->x = x;
     event->y = y;
@@ -1050,18 +1065,18 @@ _ie_bridge_dom_mouse_event_free (GtkIEEmbedDOMMouseEvent *event)
     if (event->target)
         gtk_ie_embed_dom_event_target_free (event->target);
 
-    g_free (event);
+    g_slice_free (GtkIEEmbedDOMMouseEvent, event);
 }
 
 static gboolean
-_ie_bridge_emit_mouse_event (IEBridge *ie, gchar *signal_name)
+_ie_bridge_emit_mouse_event (IEBridge *ie, const gchar *signal_name)
 {
     GtkIEEmbedDOMMouseEvent *event;
     gboolean ret;
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
 
     event = _ie_bridge_create_dom_mouse_event (ie);
-    g_signal_emit_by_name (G_OBJECT (priv->widget), signal_name,
+    g_signal_emit_by_name (priv->widget, signal_name,
                            event, &ret);
     _ie_bridge_dom_mouse_event_free (event);
 
@@ -1110,7 +1125,60 @@ void
 _ie_bridge_selection_changed (IEBridge *ie)
 {
     IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
-    g_signal_emit_by_name (G_OBJECT (priv->widget), "selection-changed");
+    g_signal_emit_by_name (priv->widget, "selection-changed");
+}
+
+void
+_ie_bridge_load_favicon (IEBridge *ie)
+{
+    IEBridgePriv *priv = IE_BRIDGE_GET_PRIVATE (ie);
+    IHTMLElementCollection *collection;
+    long i, length;
+
+    if (!priv->web_browser)
+        return;
+    if (!priv->current_document)
+        return;
+
+    priv->current_document->get_links (&collection);
+    if (!collection)
+        return;
+
+    collection->get_length (&length);
+    for (i = 0; i < length; i++) {
+        IDispatch *disp = NULL;
+        IHTMLLinkElement *link_element = NULL;
+        BSTR relName = NULL;
+        gchar *utf8_name;
+        VARIANT index;
+        index.vt = VT_I4;
+        index.intVal = i;
+        collection->item (index, index, (IDispatch **) disp);
+        disp->QueryInterface (IID_IHTMLLinkElement, (void **) &link_element);
+        if (!link_element)
+            continue;
+        link_element->get_rel (&relName);
+        if (relName) {
+            utf8_name = _ie_utils_BSTR_to_utf8 (relName);
+            if (!g_ascii_strcasecmp (utf8_name, "shortcut icon") ||
+                !g_ascii_strcasecmp (utf8_name, "icon")) {
+                BSTR href = NULL;
+                link_element->get_href (&href);
+                if (href) {
+                    gchar *utf8_href;
+                    utf8_href = _ie_utils_BSTR_to_utf8 (href);
+                    /* load a favicon */
+                    SysFreeString (href);
+                    g_free (utf8_href);
+                }
+            }
+            SysFreeString (relName);
+            g_free (utf8_name);
+        }
+        disp->Release ();
+        link_element->Release ();
+    }
+    collection->Release ();
 }
 
 /*
